#!/usr/bin/env python
# Copyright (C) 2010 W. Trevor King <wking@drexel.edu>
#
# This file is part of SiteCorePy.
#
# SiteCorePy is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# SiteCorePy is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SiteCorePy.  If not, see <http://www.gnu.org/licenses/>.

"""
Sitecore Faculty Profile	Corresponding Information

Faculty Information
-------------------
Degrees*	Degrees Earned (i.e. Dr., PhD)
College*	University Graduated From
Program	Area Within Department (i.e. Anthropology)
Office*	Office Location
Office Hours	(If applicable)
Specialization*	Research Areas/Specialization
Publications	Publications Listing (If applicable)
Professional Society	(If applicable)
Academic Distinctions	(If applicable)
Research Interests	(Please use "Specialization" Field)
Personal Site	Website Address[b] (If applicable)

Personal Profile
----------------
Title*	Professional Title (i.e. Director of Publication Management)
First Name*	First Name / Middle Name
Last Name*	Last Name
Bio	Brief Bio/Profile/Blurb
Short Bio	Presentations (If applicable)
Headshot*	Photograph - Size should be width: 125px height: 150px 
Email*	Email
Phone*	Phone # - Please use this format: (111) 111-1111
Fax	Lab Name, Lab Phone (If applicable, please use "Lab Name, Lab Phone" format)
CV	Resume

* field required.

[b] You can list both "Group website" and "Personal website" under
 "Personal Site" in the Faculty Profiles.  For example:
    Group Website: <a href="http://..."">http://...</a><br/>
    Personal Website: <a href="http://...">http://...</a><br/>

If a basic text field (a field with no "editor") has more than one
item in it then separate the items with a comma (or pipe symbols:
|). For example: If a faculty member has more than one title, then
list their titles like this: Professor of Communication, Director of
Publication Management, Yearbook Advisor, etc.
"""

class AttributeHolder (object):
    fields = []

    def __init__(self, *args, **kwargs):
        for field in self.fields:
            setattr(self, field, None)
        for field,arg in zip(self.fields, args):
            setattr(self, field, arg)
        for field,value in kwargs.items():
            assert field in self.fields, '%s not in %s' % (field, self.fields)
            setattr(self, field, value)

    def __str__(self):
        return self.__unicode__()

    def __repr__(self):
        return '<%s>' % self.__unicode__().replace('\n',';')

    def __unicode__(self):
        lines = ['%s:' % self.__class__.__name__]
        for field in self.fields:
            value = getattr(self, field)
            if value == None:
                continue
            if isinstance(value, (tuple, list)):
                for v in value:
                    lines.extend(self._format_value(field, v))
            else:
                lines.extend(self._format_value(field, value))
        return u'\n'.join(lines)

    def _format_value(self, field, value):
        value_lines = unicode(value).splitlines()
        if isinstance(value, AttributeHolder):
            prefix = '  '
        else:
            prefix = '  %s: ' % field
        return ['%s%s' % (prefix, L) for L in value_lines]

class Name (AttributeHolder):
    fields = ['first_middle', 'last']

class Graduation (AttributeHolder):
    fields = ['college', 'field', 'title', 'year']

class Contact (AttributeHolder):
    fields = ['office', 'email', 'website', 'group_site',
              'phone', 'lab', 'lab_phone']

class Bio (AttributeHolder):
    fields = ['category', 'specialization', 'publications', 'profile', 'cv']

class Professor (AttributeHolder):
    fields = ['name', 'title', 'graduations', 'contact', 'bio']

    def degrees(self):
        if self.graduations in [None, []]:
            return None
        degrees = [g.title for g in self.graduations]
        masks = {
            'M.S.':['B.S.', 'B.A.', 'B.M.'],
            'Ph.D.':['M.S.', 'M.A.', 'M.Phil.'],
            }
        # Princeton uses 'S.M.', 'A.B.', ... instead of 'M.S.', 'B.A.', ... 
        def princeton_title(title):
            if m[:2] in ['B.', 'M.']:
                fields = title.split('.')
                assert (len(fields) == 3 and fields[2] == ''), \
                    '%s -> %s\n%s' % (title, fields, self.name)
                return '.'.join([fields[1], fields[0]] + fields[2:])
            return title
        for title,masked in masks.items():
            ms = list(masked)
            for m in ms:
                pt = princeton_title(m)
                if pt != m and pt not in masked:
                    masked.append(pt)
            pt = princeton_title(title)
            if pt not in masks:
                masks[pt] = masked
        masks['Ph.D.'].extend(masks['M.S.'])
        masks['M.A'] = masks['M.Phil'] = masks['M.S.']
        for title,masked in masks.items():        
            if title in degrees:
                for m in masked:
                    if m in degrees:
                        degrees.remove(m)
        return '|'.join(degrees)

    def colleges(self):
        titles = self.degrees().split('|')
        return '|'.join([g.college for g in self.graduations
                         if g.title in titles])

    def program(self):
        "Sub-field(s)"
        if self.contact.group_site == None:
            return 'Physics'
        #TODO: group to sub-field logic

    def sites(self):
        if self.contact.group_site == None:
            return ('<a href="%s">%s</a>'
                    % (self.contact.website, self.contact.website))
        return ('Group Website: <a href="%s">%s</a>|Personal Website: <a href="%s">%s</a>'
                % (self.contact.group_site, self.contact.group_site,
                   self.contact.website, self.contact.website))

    def lab_contact(self):
        "Lab Name, Lab Phone"
        if self.contact.lab != None:
            assert self.contact.lab_phone != None, unicode(self.name)
            return '%s, %s' % (self.contact.lab, self.contact.lab_phone)
        assert self.contact.lab_phone == None, unicode(self.name)
        return None

    def profile(self):
        if self.bio.profile == None:
            return None
        return ''.join(['<p>\n%s\n</p>\n' % p
                        for p in self.bio.profile.split('\n')])
