#!/usr/bin/env python
# Copyright (C) 2010 W. Trevor King <wking@drexel.edu>
#
# This file is part of SiteCorePy.
#
# SiteCorePy is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# SiteCorePy is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SiteCorePy.  If not, see <http://www.gnu.org/licenses/>.

"""Move Professors from YAML -> Sitecore.

Professors will be created in:
  Content -> Home -> physics -> contact -> facultyDirectory
"""

import re
import time

from selenium.common.exceptions import NoSuchElementException
import yaml

from .. import SiteCoreConnection
from . import Name, Graduation, Contact, Bio, Professor


class ProfessorAdder (object):
    """See doc/faculty.txt for Drexel's field/format policy.
    """
    def __init__(self, sitecore_connection):
        self.s = sitecore_connection
        self.regexps = {
            'Degrees':re.compile('Degrees:.*', re.DOTALL),
            'College':re.compile('College:.*', re.DOTALL),
            'Program':re.compile('Program:.*', re.DOTALL),
            'Office':re.compile('Office:.*', re.DOTALL),
            'Specialization':re.compile('.*Specialization:.*', re.DOTALL),
            'Research Interests':re.compile('.*Research Interests:.*',
                                            re.DOTALL),
            'Personal Site':re.compile('.*Personal Site:.*', re.DOTALL),
            'Title':re.compile('.*Title:.*', re.DOTALL),
            'First Name':re.compile('.*First Name:.*', re.DOTALL),
            'Last Name':re.compile('.*Last Name:.*', re.DOTALL),
            'Bio':re.compile('.*Bio:.*', re.DOTALL),
            'Headshot':re.compile('.*Headshot:.*', re.DOTALL),
            'Email':re.compile('.*Email:.*', re.DOTALL),
            'Phone':re.compile('.*Phone:.*', re.DOTALL),
            'Fax':re.compile('.*Fax:.*', re.DOTALL),
            'Page Title':re.compile('.*Page Title -.*', re.DOTALL),
            'Menu Title':re.compile('.*Menu Title -.*', re.DOTALL),
            'Breadcrumb Title':re.compile('.*Breadcrumb Title -.*', re.DOTALL),
            'See Also Title':re.compile('.*See Also title -.*', re.DOTALL),
            }

    def setup(self):
        #self.s.expand_nav_section('sitecore')
        #self.s.expand_nav_section('Content')
        self.s.expand_nav_section('Home')
        for i in range(5): # 'physics' takes a while to load
            try:
                self.s.expand_nav_section('physics')
            except NoSuchElementException, e:
                time.sleep(self.s.wait_time)
        self.s.expand_nav_section('contact')
        self.s.expand_nav_section('facultyDirectory')

    def __call__(self, prof):
        name = '%s %s' % (prof.name.first_middle, prof.name.last)
        self.create_prof_page(name)
        self.lock_section(name)
        settings = [
            ('Degrees', prof.degrees()),
            ('College', prof.colleges()),
            ('Program', prof.program()),
            ('Office', prof.contact.office),
            ('Specialization', prof.bio.specialization),
            ('Research Interests', ''),
            ('Personal Site', prof.sites()),
            ('Title', prof.title),
            ('First Name', prof.name.first_middle),
            ('Last Name', prof.name.last),
            ('Bio', prof.profile()),
            ('Headshot', ('/Images/physics/headshots/%s'
                           % name.lower().replace('.','').replace(' ','_'))),
            ('Email', prof.contact.email),
            ('Phone', prof.contact.phone),
            ('Fax', prof.lab_contact()),
            ('Page Title', name),
            ('Menu Title', name),
            ('Breadcrumb Title', name),
            ('See Also Title', name),
            ]
        for field_name,value in settings:
            if value == None:
                value = ''
            granddad,field = self.s.find_field(self.regexps[field_name],
                                               field_name)
            try:
                editor_link = granddad.find_element_by_link_text('Edit Html')
                self.set_editor_field(editor_link, value)
            except NoSuchElementException: # basic text field
                field.clear()
                field.send_keys(value)
        #granddad,field = self.s.find_field('Headshot:')
        #granddad.find_element_by_link_text('Properties')
        # TODO, set width/height
        self.s.publish_section('Added/updated Prof. %s' % prof.name.last)

    def create_prof_page(self, name):
        self.s.open_nav_section(name)
        return
        self.s.open_nav_section('Copy of Shyamalendu Bose')
        old_windows = self.s.w.get_window_handles()
        self.s.w.find_element_by_link_text('Copy To').click()
        time.sleep(self.s.wait_time)
        windows = self.s.w.get_window_handles()
        current_window = self.s.w.get_current_window_handle()
        popup = [w for w in windows if w not in old_windows][0]
        if current_window != popup:
            self.s.logger.info('handling copy popup %s (from %s, old %s)'
                               % (popup, windows, current_window))
            self.s.w.switch_to_window(popup)
        name = self.s.w.find_element_by_id('Filename')
        name.clear()
        name.send_keys(
            '/sitecore/content/Home/physics/contact/facultyDirectory/%s'
            % name)
        self.s.w.find_element_by_id('OK').click()
        time.sleep(self.s.wait_time)
        current_window = self.s.w.get_current_window_handle()
        self.s.logger.info('handled copy popup %s, back to %s'
                           % (popup, current_window))
        self.s.open_nav_section(name)

    def lock_section(self, name):
        try:
            self.s.lock_section()
        except NoSuchElementException, e:
            self.s.logger.info("can't lock %s (already locked?), skipping" % name)

    def set_editor_field(self, editor_link, value):
        pass  # TODO, save first.


def main(argv):
    import optparse
    usage = """%prog [options] PROF_FILE

Where PROF_FILE is a YAML file containing professor data.

Example setup before running:
  Xvfb :99 > /dev/null 2>&1 &
  export DISPLAY=:99
  java -jar selenium-server-1.0.2-SNAPSHOT-standalone.jar > /dev/null 2>&1 &
  ./sc.py prof-import profs.yaml
"""
    p = optparse.OptionParser(usage)
    p.add_option('-u', '--url', metavar='URL', dest='url',
                 help='set the website URL (%default)',
                 default='https://webedit.drexel.edu/sitecore/login')
    p.add_option('-v', '--verbose', dest='verbose', action='count',
                 help='increment verbosity (%default)',
                 default=0)

    options,args = p.parse_args(argv[1:])
    prof_file = args[0]
    profs = yaml.load(open(prof_file, 'r'))

    s = SiteCoreConnection(options.url, options.verbose)
    s.start()
    try:
        s.login()
        add = ProfessorAdder(s)
        add.setup()
        for prof in profs:
            add(prof)
    finally:
        s.stop()
